SUBROUTINE TOVINI(RHOVAR)

!   ============================================================
!   Purpose : This program solve the TOV equations in isotropic coordinates 
!             used to provide initial conditions for the program spherical
!             Uses shooting method to find the convergent solution
!             EoS is in system.f90
!             ds^2 = -Exp(nu)*dt^2 + Exp(mu)*( dr^2 + R^2*dOmega )
!   ============================================================
!
! Parameters for the grid - set in module system - computed in sherical
! R = radial grid points (+ boundaries) ; DR = increments
!
! In input  - set in module system
! RHOVAR = central density in adimasional units
!
! In output  - set in module system
! RHOTV,PRTV,ETV = density,pressure and energy density of the TOV solution 
! MU,NU = metric exponents of the TOV metric solution
!
! Parameters
! RHOCENT,PCENT,ECENT = central density, pressure and energy density
!
!   ============================================================

  USE SYSTEMXNS

  INTEGER,PARAMETER :: MAXIT=20
  REAL,PARAMETER :: CONV=1.5D-15
  REAL ::TOL,X0,X1,X2

  REAL :: RHOCENT,PCENT,ECENT,RHOSURF,ESURF,PSURF
  REAL :: MU0,MUIN,RINI

  INTEGER :: J
  REAL :: RHOX, EX, XSUR
  REAL :: PHI2,PHI3,COEFF,COEFFN,DCOEFDMU,NUR
  REAL,DIMENSION(4) :: Y,YIN

  ! Chose central density 
  RHOCENT=RHOVAR

  ! Derive central pressure 
  TOL=1.E-8
  X0=1.
  X1=1.E-8
  X2=10.*RHOCENT
  PCENT = RTSAFEG(X0,X1,X2,TOL,RHOVAR,FUNCD_EOS)
  IF(TOL .EQ. -1.) THEN
     WRITE(6,*)"Warning from TOVINI: RTSAFEG failure, root must be bracketed"
     STOP
  END IF 
  ! Derive central energy density
  ECENT=RHOCENT+PCENT/(GAMMA-1.)

  ! Derive the surface density
  PSURF=1.E-12
  CALL EOSTOV(PSURF,RHOSURF,ESURF)

  !Chose guess for mu(0) and initial starting point to avoid singularity
  MUIN=3. 
  RINI=0.0001
  
  IF(.NOT. STRETCH) THEN 
  MIDGRID = NR/2.
  ELSE
    IF (RREG .lt. RMAX*0.5) THEN
    MIDGRID = INT( NRREG + LOG(1 + NRREG*(RMAx*0.5-RREG)*(STRR-1.)/(STRR*RREG))/LOG(STRR) )
    ELSE
    MIDGRID = INT( NRREG*RMAX*0.5/RREG)
    ENDIF
  ENDIF
  
  DO J=1,MAXIT
     ! First loop on the guess
     ! ----------------------------------------
     ! Derive starting point
     MU0=MUIN
     CALL INIT(MU0,PCENT,ECENT,RINI,Y(1:4))
     YIN(1:4)=Y(1:4)
     CALL RK4(YIN,4,RINI,DR(1)/2.-RINI,Y)
     CALL EOSTOV(Y(3),RHOX,EX)
     RHOTV(1)= RHOX
     ETV(1)  = EX
     PRTV(1) = Y(3)
     MU(1) = Y(1)
     NU(1) = Y(4)
     
     ! Integrate
     DO I=1,NR-1
        YIN(1:4)=Y(1:4)
        CALL RK4(YIN,4,R(I),DR(I),Y)
        CALL EOSTOV(Y(3),RHOX,EX)
        RHOTV(I+1) = RHOX
        ETV(I+1)   = EX
        IF(RHOX .GT. RHOSURF)THEN
           XSUR=R(I+1)
           ISUR=I+1
        END IF
        PRTV(I+1) = Y(3)
        MU(I+1) = Y(1)
        NU(I+1) = Y(4)
     END DO

     ! Convergence requires solution to be asimptotically 
     ! the Schw solution in isotrop coord.
     PHI3=(EXP(MU(NR)/4.)-1.)
     PHI2=(EXP(MU(MIDGRID)/4.)-1.)
     COEFF=PHI3*R(NR)-(PHI2*R(MIDGRID))

     IF(ISUR+10 .GT. MIDGRID-1)THEN
        WRITE(6,*)"Warning from TOVINI:NS surf too close to outer boundary"
        STOP
     END IF
     
     IF(ABS(COEFF) .LT. CONV)EXIT

     ! Second loop on the guess
     ! ----------------------------------
     ! Derive starting point
     MU0=MUIN*1.00001
     CALL INIT(MU0,PCENT,ECENT,RINI,Y(1:4))
     YIN(1:4)=Y(1:4)
     CALL RK4(YIN,4,RINI,DR(1)/2.-RINI,Y)
     CALL EOSTOV(Y(3),RHOX,EX)
     RHOTV(1)= RHOX
     ETV(1)  = EX
     PRTV(1) = Y(3)
     MU(1) = Y(1)
     NU(1) = Y(4)
     
     ! Integrate
     DO I=1,NR-1
        YIN(1:4)=Y(1:4)
        CALL RK4(YIN,4,R(I),DR(I),Y)
        CALL EOSTOV(Y(3),RHOX,EX)
        RHOTV(I+1)= RHOX
        ETV(I+1)  = EX
        PRTV(I+1) = Y(3)
        MU(I+1) = Y(1)
        NU(I+1) = Y(4)
     END DO
     
     ! Convergence requires solution to be asimptotically 
     ! the Schw solution in isotrop coord.
     PHI3=(EXP(MU(NR)/4.)-1.)
     PHI2=(EXP(MU(MIDGRID)/4.)-1.)
     COEFFN=PHI3*R(NR)-(PHI2*R(MIDGRID))
     
     DCOEFDMU=(COEFFN-COEFF)/MUIN/0.00001
     IF(VERBOSE)WRITE(6,*)'Conv = ',COEFF,' Mu0 = ',MUIN
     MUIN=MAX(MUIN-COEFF/DCOEFDMU,0.01)
     
  END DO

  NUR=NU(NR)
  NU(1:NR)=NU(1:NR)-NUR+2.*LOG((1.-PHI3)/(1+PHI3))

  IF(VERBOSE)THEN
     WRITE(6,*)''
     WRITE(6,*)'Mass        = ',PHI3*R(NR)*2
     WRITE(6,*)'Iso. Radius = ',R(ISUR)
     WRITE(6,*)'Sch. Radius = ',R(ISUR)*EXP(MU(ISUR)/2.)
     WRITE(6,*)'Phi_c = ',EXP(MU(1)/4.),' Alpha_c = ',EXP(NU(1)/2.)
  END IF

  IF(CHUP.AND.(WRT.OR.WRTF))THEN
     OPEN(12,FILE=trim(adjustl(subdirpath))//"TOVini.dat") !FOLDER
     WRITE(12,*)NR,R(ISUR)
     DO I=1,NR
        WRITE(12,*)R(I),MU(I),RHOTV(I),NU(I),PRTV(I),ETV(I)
     END DO
     CLOSE(12)
  END IF

  DO IX=1,NTH
     WSURF(IX)=ISUR
  ENDDO
  
END SUBROUTINE TOVINI

! ********************************************************
! ********************************************************

SUBROUTINE RK4(Y,N,X,H,YOUT)

!   ============================================================
!   Purpose : 4th order RK integrator (from NR)
!   ============================================================

  INTEGER N,NMAX
  REAL H,X,DYDX(N),Y(N),YOUT(N)
  PARAMETER (NMAX=4)
  INTEGER I
  REAL H6,HH,XH,DYM(NMAX),DYT(NMAX),YT(NMAX)
  HH=H*0.5
  H6=H/6.
  XH=X+HH
  CALL DERIVS(X,Y,DYDX)
  DO I=1,N
     YT(I)=Y(I)+HH*DYDX(I)
  END DO
  CALL DERIVS(XH,YT,DYT)
  DO  I=1,N
     YT(I)=Y(I)+HH*DYT(I)
  END DO
  CALL DERIVS(XH,YT,DYM)
  DO I=1,N
     YT(I)=Y(I)+H*DYM(I)
     DYM(I)=DYT(I)+DYM(I)
  END DO
  CALL DERIVS(X+H,YT,DYT)
  DO I=1,N
     YOUT(I)=Y(I)+H6*(DYDX(I)+DYT(I)+2.*DYM(I))
  END DO
  RETURN

END SUBROUTINE RK4

! ********************************************************
! ********************************************************

SUBROUTINE DERIVS(X,Y,DY)

!   ============================================================
!   Purpose : Derivative of TOV eq in isotrop. coord.
!   ============================================================

  USE SYSTEMXNS
  
  REAL,DIMENSION(4) :: Y,DY
  REAL :: X,RHOX,ENX,ETOT
  
  CALL EOSTOV(Y(3),RHOX,ENX)
  ETOT=RHOX+ENX
  
  DY(1)=Y(2)
  DY(2)=-8.*PI*ETOT*EXP(Y(1))-2./X*Y(2)-0.25*Y(2)**2
  DY(3)=-(Y(3)+ETOT)*(32*PI*EXP(Y(1))*X*Y(3)-Y(2)*(4.+X*Y(2)))/(8.+4*X*Y(2))
  DY(4)=(8.*PI*Y(3)*EXP(Y(1))-0.25*Y(2)**2-1./X*Y(2))/(Y(2)/2.+1./X)
  
  RETURN

END SUBROUTINE DERIVS

! ********************************************************
! ********************************************************

SUBROUTINE INIT(MU0,PCENT,ECENT,XINI,YINI)

!   ============================================================
!   Purpose : Initial solution of TOV equations computed in Rini 
!             using Taylor expansion
!   ============================================================

  USE SYSTEMXNS
  
  REAL,DIMENSION(4) :: YINI
  REAL :: XINI,P0,RHO0,MU1,PR1,NU1,MU0,ECENT,PCENT
  
  RHO0=ECENT
  P0=PCENT
  
  MU1 = -4./3.*EXP(MU0)*PI*RHO0
  PR1 = -(P0+RHO0)*(32*EXP(MU0)*PI*P0-8*MU1)/8.
  NU1 = +(8*EXP(MU0)*PI*P0+2*MU1)/2.
  
  YINI(1) = MU0+MU1*XINI**2
  YINI(2) = 2*XINI*MU1
  YINI(3) = P0+PR1*XINI**2
  YINI(4) = NU1*XINI**2
  
  RETURN

END SUBROUTINE INIT

! ********************************************************
! ********************************************************
